@@ -9,7 +9,9 @@ cran_pkg_dates = function(full = FALSE, maintainer = 'Yihui Xie') {
     i = which(x == '<td>Published:</td>')
     if (length(i) == 0) stop('Cannot find the publishing date from ', u)
     d = as.Date(gsub('</?td>', '', x[i[1] + 1]))
-    x = try(readLines(u <- sprintf('https://cran.r-project.org/src/contrib/Archive/%s/', p)))
+    x = try_silent(suppressWarnings(readLines(
+      u <- sprintf('https://cran.r-project.org/src/contrib/Archive/%s/', p)
+    )))
     if (inherits(x, 'try-error')) {
       info[[p]] = d; next
     }
@@ -26,7 +28,28 @@ cran_updatable = function(days = 90, maintainer = 'Yihui Xie') {
   flag = unlist(lapply(info, function(d) {
     sum(d > Sys.Date() - 180) < 6 && d[1] < Sys.Date() - days
   }))
-  names(which(flag))
+  if (length(pkgs <- names(which(flag))) == 0) return(pkgs)
+  # look into DESCRIPTION in Github repos and see if new version has been pushed
+  info = tools::CRAN_package_db()
+  info = info[info$Package %in% pkgs, , drop = FALSE]
+  pkgs = info$Package
+  for (i in seq_len(nrow(info))) {
+    b = grep_sub('^(https://github.com/[^/]+/[^/]+)/issues$', '\\1', info$BugReports[i])
+    if (length(b) != 1) next
+    f = tempfile()
+    u = paste0(b, '/raw/HEAD/DESCRIPTION')
+    if (is.null(tryCatch(download.file(u, f, quiet = TRUE), error = function(e) NULL))) next
+    d = read.dcf(f)
+    file.remove(f)
+    if (!'Version' %in% colnames(d)) next
+    if (as.numeric_version(d[, 'Version']) <= paste0(info$Version[i], '.1')) {
+      pkgs = setdiff(pkgs, info$Package[i])
+      message('Skipped package ', info$Package[i], ' ', d[, 'Version'], ' (no new version).')
+    } else {
+      message('Package can be updated: ', b)
+    }
+  }
+  pkgs
 }
 
 
@@ -56,8 +79,8 @@ check_package_name = function() {
 #' @rdname is_R_CMD_check
 #' @export
 check_old_package = function(name, version) {
-  if (is.na(pkg <- check_package_name()) || pkg != name) return(FALSE)
-  tryCatch(packageVersion(name) <= version, error = function(e) FALSE)
+  if (is.na(pkg <- check_package_name()) || !(pkg %in% name)) return(FALSE)
+  tryCatch(packageVersion(pkg) <= version[pkg == name], error = function(e) FALSE)
 }
 
 # return package maintainers (with email addresses)
@@ -73,15 +96,15 @@ pkg_maintainers = function(pkgs) {
 #' @param file The path to the source package tarball. By default, the current
 #'   working directory is treated as the package root directory, and
 #'   automatically built into a tarball, which is deleted after submission. This
-#'   means you should run \code{xfun::submit_cran()} in the root directory of a
+#'   means you should run `xfun::submit_cran()` in the root directory of a
 #'   package project, unless you want to pass a path explicitly to the
-#'   \code{file} argument.
+#'   `file` argument.
 #' @param comment Submission comments for CRAN. By default, if a file
 #'   \file{cran-comments.md} exists, its content will be read and used as the
 #'   comment.
-#' @seealso \code{devtools::submit_cran()} does the same job, with a few more
+#' @seealso `devtools::submit_cran()` does the same job, with a few more
 #'   dependencies in addition to \pkg{curl} (such as \pkg{cli});
-#'   \code{xfun::submit_cran()} only depends on \pkg{curl}.
+#'   `xfun::submit_cran()` only depends on \pkg{curl}.
 #' @export
 submit_cran = function(file = pkg_build(), comment = '') {
   # if the tarball is automatically created, delete it after submission
@@ -101,8 +124,9 @@ submit_cran = function(file = pkg_build(), comment = '') {
   }
   params = list(
     uploaded_file = curl::form_file(file), name = info[1], email = info[2],
-    comment = comment, upload = 'Upload package'
+    upload = 'Upload package'
   )
+  params$comment = if (length(comment)) comment
   server = 'https://xmpalantir.wu.ac.at/cransubmit/index2.php'
 
   # submit the form
